@@ -7,6 +7,6 @@
 #define DLLEXPORT extern
 #endif
 
-#define xml6_warn(msg) fprintf(stderr, "%s:%d: %s\n", __FILE__, __LINE__, (msg));
+#define xml6_warn(msg) fprintf(stderr, __FILE__ ":%d: %s\n", __LINE__, (msg));
 
 #endif /* __XML6_H */
